@@ -12,7 +12,7 @@
 ##############################################################################
 """Access control support
 
-$Id: Role.py 67839 2006-05-02 14:22:06Z regebro $
+$Id: Role.py 73388 2007-03-20 09:02:28Z mj $
 """
 from cgi import escape
 
@@ -27,6 +27,7 @@ from zope.interface import implements
 
 from interfaces import IRoleManager
 from Permission import Permission
+from requestmethod import postonly
 
 
 DEFAULTMAXLISTUSERS=250
@@ -128,6 +129,7 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
                              help_product='OFSP')
 
     security.declareProtected(change_permissions, 'manage_role')
+    @postonly
     def manage_role(self, role_to_manage, permissions=[], REQUEST=None):
         """Change the permissions given to the given role.
         """
@@ -146,6 +148,7 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
                                  help_product='OFSP')
 
     security.declareProtected(change_permissions, 'manage_acquiredPermissions')
+    @postonly
     def manage_acquiredPermissions(self, permissions=[], REQUEST=None):
         """Change the permissions that acquire.
         """
@@ -167,6 +170,7 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
                                    help_product='OFSP')
 
     security.declareProtected(change_permissions, 'manage_permission')
+    @postonly
     def manage_permission(self, permission_to_manage,
                           roles=[], acquire=0, REQUEST=None):
         """Change the settings for the given permission.
@@ -205,6 +209,7 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
             return apply(self._normal_manage_access,(), kw)
 
     security.declareProtected(change_permissions, 'manage_changePermissions')
+    @postonly
     def manage_changePermissions(self, REQUEST):
         """Change all permissions settings, called by management screen.
         """
@@ -358,6 +363,7 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
         return tuple(dict.get(userid, []))
 
     security.declareProtected(change_permissions, 'manage_addLocalRoles')
+    @postonly
     def manage_addLocalRoles(self, userid, roles, REQUEST=None):
         """Set local roles for a user."""
         if not roles:
@@ -376,6 +382,7 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
             return self.manage_listLocalRoles(self, REQUEST, stat=stat)
 
     security.declareProtected(change_permissions, 'manage_setLocalRoles')
+    @postonly
     def manage_setLocalRoles(self, userid, roles, REQUEST=None):
         """Set local roles for a user."""
         if not roles:
@@ -390,6 +397,7 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
             return self.manage_listLocalRoles(self, REQUEST, stat=stat)
 
     security.declareProtected(change_permissions, 'manage_delLocalRoles')
+    @postonly
     def manage_delLocalRoles(self, userids, REQUEST=None):
         """Remove all local roles for a user."""
         dict=self.__ac_local_roles__
@@ -482,6 +490,7 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
 
         return self.manage_access(REQUEST)
 
+    @postonly
     def _addRole(self, role, REQUEST=None):
         if not role:
             return MessageDialog(
@@ -499,6 +508,7 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
         if REQUEST is not None:
             return self.manage_access(REQUEST)
 
+    @postonly
     def _delRoles(self, roles, REQUEST=None):
         if not roles:
             return MessageDialog(
