@@ -36,7 +36,7 @@
   USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
   DAMAGE.
 
-  $Id: cAccessControl.c,v 1.20.2.22 2005/08/02 09:32:30 andreasjung Exp $
+  $Id: cAccessControl.c 41697 2006-02-19 12:20:27Z andreasjung $
 
   If you have questions regarding this software,
   contact:
@@ -48,8 +48,8 @@
 
 */
 
-#include "ExtensionClass.h"
-#include "Acquisition.h"
+#include "ExtensionClass/ExtensionClass.h"
+#include "Acquisition/Acquisition.h"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -200,22 +200,6 @@ callfunction6(PyObject *function,
   Py_DECREF(t);
   return r;
 }
-  
-
-static int 
-unpacktuple1(PyObject *args, char *name, int min, PyObject **a0)
-{ 
-  int l;
-  l=PyTuple_Size(args);
-  if (l < 0) return -1;
-  if (l < min) 
-    {
-      PyErr_Format(PyExc_TypeError, "expected %d arguments, got %d", min, l);
-      return -1;
-    }
-  if (l > 0) *a0=PyTuple_GET_ITEM(args, 0);
-  return 0;
-}
 
 static int 
 unpacktuple2(PyObject *args, char *name, int min, 
@@ -252,6 +236,25 @@ unpacktuple3(PyObject *args, char *name, int min,
   return 0;
 }
 
+static int 
+unpacktuple4(PyObject *args, char *name, int min, 
+             PyObject **a0, PyObject **a1, PyObject **a2, PyObject **a3)
+{ 
+  int l;
+  l=PyTuple_Size(args);
+  if (l < 0) return -1;
+  if (l < min) 
+    {
+      PyErr_Format(PyExc_TypeError, "expected %d arguments, got %d", min, l);
+      return -1;
+    }
+  if (l > 0) *a0=PyTuple_GET_ITEM(args, 0);
+  if (l > 1) *a1=PyTuple_GET_ITEM(args, 1);
+  if (l > 2) *a2=PyTuple_GET_ITEM(args, 2);
+  if (l > 3) *a3=PyTuple_GET_ITEM(args, 3);
+  return 0;
+}
+
 static int 
 unpacktuple5(PyObject *args, char *name, int min, 
              PyObject **a0, PyObject **a1, PyObject **a2, 
@@ -334,18 +337,18 @@ typedef struct {
 */
 
 static PyObject *ZopeSecurityPolicy_validate(PyObject *self, PyObject *args);
-static PyObject *ZopeSecurityPolicy_checkPermission(PyObject *self,
-	PyObject *args);
 static void ZopeSecurityPolicy_dealloc(ZopeSecurityPolicy *self);
 
 
 static PyObject *PermissionRole_init(PermissionRole *self, PyObject *args);
-static PyObject *PermissionRole_of(PermissionRole *self, PyObject *args);
+static PyObject *PermissionRole_of(PermissionRole *self, PyObject *parent);
+static PyObject *
+PermissionRole_rolesForPermissionOn(PermissionRole *self, PyObject *value);
 static void PermissionRole_dealloc(PermissionRole *self);
 
 static PyObject *PermissionRole_getattro(PermissionRole *self, PyObject *name);
 
-static PyObject *imPermissionRole_of(imPermissionRole *self, PyObject *args);
+static PyObject *imPermissionRole_of(imPermissionRole *self, PyObject *parent);
 static int imPermissionRole_length(imPermissionRole *self);
 static PyObject *imPermissionRole_get(imPermissionRole *self,
 	int item);
@@ -362,8 +365,6 @@ static PyObject *permissionName(PyObject *name);
 
 static PyObject *SecurityManager_validate(SecurityManager *self, 
                                           PyObject *args);
-static PyObject *SecurityManager_validateValue(SecurityManager *self,
-                                               PyObject *args);
 static PyObject *SecurityManager_DTMLValidate(SecurityManager *self,
                                               PyObject *args);
 static PyObject *SecurityManager_checkPermission(SecurityManager *self, 
@@ -413,11 +414,6 @@ static PyMethodDef ZopeSecurityPolicy_methods[] = {
 		METH_VARARGS,
 		""
 	},
-	{"checkPermission",
-		(PyCFunction)ZopeSecurityPolicy_checkPermission,
-		METH_VARARGS,
-		""
-	},
 	{ NULL, NULL }
 };
 
@@ -455,7 +451,7 @@ static PyExtensionClass ZopeSecurityPolicyType = {
 	NULL,					/* tp_next	*/
 #endif
 	METHOD_CHAIN(ZopeSecurityPolicy_methods),/* methods	*/
-	EXTENSIONCLASS_BINDABLE_FLAG,		/* flags	*/
+	(void *)(EXTENSIONCLASS_BINDABLE_FLAG),		/* flags	*/
 };
 
 
@@ -472,11 +468,6 @@ static PyMethodDef SecurityManager_methods[] = {
 		METH_VARARGS,
 		""
 	},
-	{"validateValue",
-		(PyCFunction)SecurityManager_validateValue,
-		METH_VARARGS,
-		""
-	},
 	{"checkPermission",
 		(PyCFunction)SecurityManager_checkPermission,
 		METH_VARARGS,
@@ -533,7 +524,12 @@ static PyMethodDef PermissionRole_methods[] = {
 	},
 	{"__of__",
 		(PyCFunction)PermissionRole_of,
-		METH_VARARGS,
+		METH_O,
+		""
+	},
+	{"rolesForPermissionOn",
+		(PyCFunction)PermissionRole_rolesForPermissionOn,
+		METH_O,
 		""
 	},
 	{ NULL, NULL }
@@ -573,10 +569,10 @@ static PyExtensionClass PermissionRoleType = {
 	NULL,					/* tp_next	*/
 #endif
 	METHOD_CHAIN(PermissionRole_methods),	/* methods	*/
-	EXTENSIONCLASS_BINDABLE_FLAG/*|
+	(void *)(EXTENSIONCLASS_BINDABLE_FLAG)/*|
 	EXTENSIONCLASS_INSTDICT_FLAG*/,		/* flags	*/
 	NULL,					/* Class dict	*/
-	NULL,					/* bases	*/
+	0,					/* bases	*/
 	NULL,					/* reserved	*/
 };
 
@@ -585,7 +581,12 @@ static char imPermissionRole__doc__[] = "imPermissionRole C implementation";
 static PyMethodDef imPermissionRole_methods[] = {
 	{"__of__",
 		(PyCFunction)imPermissionRole_of,
-		METH_VARARGS,
+		METH_O,
+		""
+	},
+	{"rolesForPermissionOn",
+		(PyCFunction)imPermissionRole_of,
+		METH_O,
 		""
 	},
 	{ NULL, NULL }
@@ -638,7 +639,7 @@ static PyExtensionClass imPermissionRoleType = {
 	NULL,					/* tp_next	*/
 #endif
 	METHOD_CHAIN(imPermissionRole_methods), /* methods	*/
-	EXTENSIONCLASS_BINDABLE_FLAG,		/* flags	*/
+	(void *)(EXTENSIONCLASS_BINDABLE_FLAG),		/* flags	*/
 };
 
 
@@ -650,12 +651,12 @@ static PyExtensionClass imPermissionRoleType = {
 static PyObject *Containers = NULL;
 static PyObject *ContainerAssertions = NULL;
 static PyObject *Unauthorized = NULL;
-static PyObject *LOG = NULL;
-static PyObject *PROBLEM = NULL;
+static PyObject *warn= NULL;
 static PyObject *NoSequenceFormat = NULL;
 static PyObject *_what_not_even_god_should_do = NULL;
 static PyObject *Anonymous = NULL;
 static PyObject *AnonymousTuple = NULL;
+static PyObject *ManagerTuple = NULL;
 static PyObject *imPermissionRoleObj = NULL;
 static PyObject *defaultPermission = NULL;
 static PyObject *__roles__ = NULL;
@@ -674,6 +675,7 @@ static PyObject *unrestrictedTraverse_str = NULL;
 static PyObject *aq_validate = NULL;
 static PyObject *aq_parent_str = NULL;
 static PyObject *_check_context_str = NULL;
+static PyObject *getRoles = NULL;
 static PyObject *getWrappedOwner_str = NULL;
 
 static int ownerous = 1;
@@ -702,7 +704,8 @@ ZopeSecurityPolicy_setup(void) {
 	UNLESS (__roles__ = PyString_FromString("__roles__")) return -1;
 	UNLESS (__of__ = PyString_FromString("__of__")) return -1;
 	UNLESS (Anonymous = PyString_FromString("Anonymous")) return -1;
-	UNLESS (AnonymousTuple = Py_BuildValue("(s)", "Anonymous")) return -1;
+	UNLESS (AnonymousTuple = Py_BuildValue("(O)", Anonymous)) return -1;
+	UNLESS (ManagerTuple = Py_BuildValue("(s)", "Manager")) return -1;
 	UNLESS (stack_str = PyString_FromString("stack")) return -1;
 	UNLESS (user_str = PyString_FromString("user")) return -1;
 	UNLESS (validate_str = PyString_FromString("validate")) return -1;
@@ -752,8 +755,7 @@ static void unauthErr(PyObject *name, PyObject *value) {
 ** ZopeSecurityPolicy_validate
 */
 
-static PyObject *
-ZopeSecurityPolicy_validate(PyObject *self, PyObject *args) {
+static PyObject *ZopeSecurityPolicy_validate(PyObject *self, PyObject *args) {
 	PyObject *accessed = NULL;  /* Note: accessed is not used. */
 	PyObject *container = NULL;
 	PyObject *name = NULL;
@@ -773,6 +775,7 @@ ZopeSecurityPolicy_validate(PyObject *self, PyObject *args) {
 	PyObject *method = NULL;
 	PyObject *tmp = NULL;
 
+
         int i, l, contains;
         PyObject *r;
 
@@ -812,7 +815,6 @@ ZopeSecurityPolicy_validate(PyObject *self, PyObject *args) {
 		return NULL;  /* roles is not owned yet */
 	    }
 	}
-
 	Py_XINCREF(roles);	/* Convert the borrowed ref to a real one */
 
 	/*| containerbase = aq_base(container)
@@ -837,11 +839,15 @@ ZopeSecurityPolicy_validate(PyObject *self, PyObject *args) {
 	**| # the object
 	**|
 	**| if roles is _noroles:
-	**|    roles = getattr(value, "__roles__", _noroles)
+	**|    roles = getRoles(container, name, value, _noroles)
 	*/
 
         if (roles == NULL) {
-          roles = PyObject_GetAttr(value, __roles__);
+	  roles = callfunction4(getRoles, container, name, value, getRoles);
+	  if (roles == getRoles) {
+	    Py_DECREF(roles);
+            roles = NULL;
+	  }
           if (roles == NULL)
             PyErr_Clear();
 	}
@@ -1008,7 +1014,7 @@ ZopeSecurityPolicy_validate(PyObject *self, PyObject *args) {
 	**| try:
 	**|    if roles is None or 'Anonymous' in roles: return 1
 	**| except TypeError:
-	**|     LOG("Zope Security Policy", PROBLEM, '"%s' passed as roles"
+	**|     LOG.warn('"%s' passed as roles"
 	**|		" during validation of '%s' is not a sequence." % 
 	**|		('roles', name))
 	**|	raise
@@ -1038,8 +1044,7 @@ ZopeSecurityPolicy_validate(PyObject *self, PyObject *args) {
               m=PyObject_Repr(roles);
               if (m) ASSIGN(m, Py_BuildValue("OO", m, name));
               if (m) ASSIGN(m, PyString_Format(NoSequenceFormat, m));
-              if (m) ASSIGN(m, PyObject_CallFunction(LOG, "sOO",
-                     "Zope Security Policy", PROBLEM, m)); 
+              if (m) ASSIGN(m, PyObject_CallFunction(warn, "O", m));
               Py_XDECREF(m);
               PyErr_Restore(t, v, tb);
               goto err;
@@ -1065,7 +1070,8 @@ ZopeSecurityPolicy_validate(PyObject *self, PyObject *args) {
 	**|    if (owner is not None) and not owner.allowed(value, roles)
 	**| 	  # We don't want someone to acquire if they can't 
 	**|	  # get an unacquired!
-	**|       raise Unauthorized(name, value)
+	**|       raise Unauthorized, ('You are not authorized to'
+	**|	      'access <em>%s</em>.' % cleanupName(name, value))
 	*/
 
 		eo = PySequence_GetItem(stack, -1);
@@ -1264,69 +1270,6 @@ ZopeSecurityPolicy_validate(PyObject *self, PyObject *args) {
 }
 
 
-/*
-** ZopeSecurityPolicy_checkPermission
-**
-*/
-
-static PyObject *ZopeSecurityPolicy_checkPermission(PyObject *self,
-	PyObject *args) {
-
-	PyObject *permission = NULL;
-	PyObject *object = NULL;
-	PyObject *context = NULL;
-	PyObject *roles;
-	PyObject *result = NULL;
-	PyObject *user;
-
-	/*| def checkPermission(self, permission, object, context)
-	*/
-
-	if (unpacktuple3(args, "checkPermission", 3, 
-                         &permission, &object, &context) < 0)
-		return NULL;
-
-	/*| roles = rolesForPermissionOn(permission, object)
-	*/
-
-	roles = c_rolesForPermissionOn(self, permission, object, OBJECT(NULL));
-	if (roles == NULL)
-          return NULL;
-
-	/*| if type(roles) in (StringType, UnicodeType):
-	**|	roles = [roles]
-	*/
-
-	if ( PyString_Check(roles) || PyUnicode_Check(roles) ) {
-          PyObject *r;
-
-          r = PyList_New(1);
-          if (r == NULL) {
-            Py_DECREF(roles);
-            return NULL;
-          }
-          /* Note: ref to roles is passed to the list object. */
-          PyList_SET_ITEM(r, 0, roles);
-          roles = r;
-	}
-
-	/*| return context.user.allowed(object, roles)
-	*/
-
-	user = PyObject_GetAttr(context, user_str);
-	if (user != NULL) {
-          ASSIGN(user, PyObject_GetAttr(user, allowed_str));
-          if (user != NULL) {
-            result = callfunction2(user, object, roles);
-            Py_DECREF(user);
-          }
-	}
-
-	Py_DECREF(roles);
-
-	return result;
-}
-
 /*
 ** ZopeSecurityPolicy_dealloc
 **
@@ -1375,23 +1318,6 @@ SecurityManager_validate(SecurityManager *self, PyObject *args)
                        accessed, container, name, value, self->context, roles);
 }
 
-static PyObject *
-SecurityManager_validateValue(SecurityManager *self, PyObject *args)
-{
-  PyObject *value=Py_None, *roles=NULL;
-  
-  if (unpacktuple2(args, "validateValue", 1, &value, &roles) < 0) return NULL;
-          
-  CHECK_SECURITY_MANAGER_STATE(self, NULL);
-  GET_SECURITY_MANAGER_VALIDATE(self, NULL);
-  
-  if (roles==NULL) 
-    return callfunction5(self->validate, 
-                         Py_None, Py_None, Py_None, value, self->context);
-  return callfunction6(self->validate, 
-                       Py_None, Py_None, Py_None, value, self->context, roles);
-}
-
 static PyObject *
 SecurityManager_DTMLValidate(SecurityManager *self, PyObject *args)
 {
@@ -1515,6 +1441,7 @@ SecurityManager_setattro(SecurityManager *self, PyObject *name, PyObject *v)
 
 
 
+
 /*
 ** PermissionRole_init
 **
@@ -1548,22 +1475,15 @@ static PyObject *PermissionRole_init(PermissionRole *self, PyObject *args) {
 }
 
 /*
-** PermissionRole_of
-**
+    def __of__(self, parent):
 */
+static PyObject *
+PermissionRole_of(PermissionRole *self, PyObject *parent) {
 
-static PyObject *PermissionRole_of(PermissionRole *self, PyObject *args) {
-
-	PyObject *parent = NULL;
 	imPermissionRole *r = NULL;
 	PyObject *_p = NULL;
 	PyObject *result = NULL;
 
-	/*|def __of__(self, parent):
-	*/
-
-	if (unpacktuple1(args,"__of__", 1, &parent) < 0) return NULL;
-
 	/*| r = imPermissionRole()
 	*/
 
@@ -1612,6 +1532,18 @@ static PyObject *PermissionRole_of(PermissionRole *self, PyObject *args) {
 	return result;
 }
 
+/*
+    def rolesForPermissionOn(self, value):
+        return rolesForPermissionOn(None, value, self._d, self._p)
+*/
+static PyObject *
+PermissionRole_rolesForPermissionOn(PermissionRole *self, PyObject *value)
+{
+  return c_rolesForPermissionOn(NULL, value, self->__roles__, self->_p);
+}
+
+
+
 /*
 ** PermissionRole_dealloc
 **
@@ -1670,202 +1602,17 @@ PermissionRole_getattro(PermissionRole *self, PyObject *name) {
 
 
 /*
-** imPermissionRole_of
-**
+    def __of__(self, value):
+        return rolesForPermissionOn(None, value, self._d, self._p)
 */
-
-static PyObject *imPermissionRole_of(imPermissionRole *self, PyObject *args) {
-
-	PyObject *parent = NULL;
-	PyObject *obj = NULL;
-	PyObject *n = NULL;
-	PyObject *r = NULL;
-	PyObject *roles = NULL;
-	PyObject *result = NULL;
-	PyObject *tobj = NULL;
-
-	/*|def __of__(self, parent):
-	**| obj = parent
-	**| n = self._p
-	**| r = None
-	*/
-
-	if (unpacktuple1(args, "__of__", 1, &parent) < 0) return NULL;
-
-	obj = parent;
-	Py_INCREF(obj);
-
-	n = self->_p;
-	if (n == NULL) {
-                /* XXX Should not be possible */
-		PyErr_SetString(PyExc_AttributeError, "_p");
-                Py_DECREF(obj);
-                return NULL;
-	}
-	Py_INCREF(n);
-
-	r = Py_None;
-	Py_INCREF(r);
-	
-	/*| while 1:
-	*/
-	
-	while (1) {
-
-	/*|    if hasattr(obj, n):
-	**|       roles = getattr(obj, n)   
-	**|
-	**|       if roles is None: return 'Anonymous', 
-	*/
-
-		roles = PyObject_GetAttr(obj, n);
-		if (roles != NULL) {
-
-			if (roles == Py_None) {
-				result = AnonymousTuple;
-				Py_INCREF(result);
-				goto err;
-			}
-
-		/*|
-		**|	  t = type(roles)
-		**|  
-		**|	  if t is TupleType:
-		**|          # If we get a tuple, then we don't acquire
-		**|	     if r is None: return roles
-		**|	     return r + list(roles)
-		*/
-			if (PyTuple_Check(roles)) {
-				if (r == Py_None) {
-					result = roles;
-					roles = NULL;	/* avoid inc/decref */
-					goto err;
-				} else {
-					PyObject *list;
-					list = PySequence_List(roles);
-                                        if (list != NULL) {
-                                          result = PySequence_Concat(r, list);
-                                          Py_DECREF(list);
-                                        }
-					goto err;
-				}
-			}
-		
-		/*|
-		**|       if t in (StringType, UnicodeType):
-		**|          # We found roles set to a name.  Start over
-		**|	     # with the new permission name.  If the permission
-		**|	     # name is '', then treat as private!
-		*/
-
-			if (PyString_Check(roles) || PyUnicode_Check(roles)) {
-
-		/*|
-		**|          if roles:
-		**|             if roles != n:
-		**|                n = roles
-		**|             # If we find a name that is the same as the
-		**|             # current name, we just ignore it.
-		**|             roles = None
-		**|          else:
-		**|             return _what_not_even_god_should_do
-		**|
-		*/
-				if (PyObject_IsTrue(roles)) {
-                                  if (PyObject_Compare(roles, n) != 0)  {
-                                    Py_DECREF(n);
-                                    n = roles;
-                                    Py_INCREF(n);
-                                  }
-                                  Py_DECREF(roles);
-                                  roles = Py_None;
-                                  Py_INCREF(roles);
-				} else {
-                                  result = _what_not_even_god_should_do;
-                                  Py_INCREF(result);
-                                  goto err;
-				}
-			} else {
-
-		/*|       elif roles:
-		**|          if r is None: r = list(roles)
-		**|          else: r = r+list(roles)
-		*/
-                          if (PyObject_IsTrue(roles)) {
-                            if (r == Py_None) {
-                              Py_DECREF(r);
-                              r = PySequence_List(roles);
-                            } else {
-                              PyObject *list;
-
-                              list = PySequence_List(roles);
-                              if (list != NULL) {
-                                ASSIGN(r, PySequence_Concat(r, list));
-                                Py_DECREF(list);
-                                if (r == NULL)
-                                  goto err;
-                              }
-                              else
-                                goto err;
-                            }
-                          }
-			}
-		}
-                else
-                  PyErr_Clear();
-
-	/*|    obj = getattr(obj, 'aq_inner', None)
-	**|    if obj is None: break
-	**|    obj = obj.aq_parent
-	*/
-
-		if (!aq_isWrapper(obj)) break;
-		tobj = aq_inner(obj);
-		if (tobj == NULL) goto err;
-		Py_DECREF(obj);
-		obj = tobj;
-
-		if (obj == Py_None) break;
-		if (!aq_isWrapper(obj)) break;
-		tobj = aq_parent(obj);
-		if (tobj == NULL) goto err;
-		Py_DECREF(obj);
-		obj = tobj;
-		
-	}	/* end while 1 */
-
-	/*|
-	**| if r is None: r = self._d
-	*/
-
-	if (r == Py_None) {
-		Py_DECREF(r);
-		r = self->__roles__;
-		if (r == NULL) goto err;
-		Py_INCREF(r);
-	}
-
-	/*|
-	**| return r
-	*/
-
-	result = r;
-	Py_INCREF(result);
-
-	err:
-
-	Py_XDECREF(n);
-	Py_XDECREF(r);
-	Py_XDECREF(obj);
-	Py_XDECREF(roles);
-
-	return result;
+static PyObject *
+imPermissionRole_of(imPermissionRole *self, PyObject *value) {
+  return c_rolesForPermissionOn(NULL, value, self->__roles__, self->_p);
 }
 
 /*
 ** imPermissionRole_length
 */
-
 static int imPermissionRole_length(imPermissionRole *self) {
 
 	int l;
@@ -1969,51 +1716,220 @@ static PyObject *rolesForPermissionOn(PyObject *self, PyObject *args) {
 	PyObject *perm = NULL;
 	PyObject *object = NULL;
 	PyObject *deflt = NULL;
+	PyObject *n = NULL;
 
 	/*|def rolesForPermissionOn(perm, object, default=('Manager',)):
 	**|
 	**| """Return the roles that have the permisson on the given object"""
 	*/
 
-	if (unpacktuple3(args, "rolesForPermissionOn", 2, 
-                         &perm, &object, &deflt) < 0)
+	if (unpacktuple4(args, "rolesForPermissionOn", 2, 
+                         &perm, &object, &deflt, &n) < 0)
 		return NULL;
-        return c_rolesForPermissionOn(self, perm, object, deflt);
+        return c_rolesForPermissionOn(perm, object, deflt, n);
 }
 
-
+/* 
+def rolesForPermissionOn(perm, object, default=_default_roles, n=None):
+    """Return the roles that have the given permission on the given object
+    """
+*/
 static PyObject *
-c_rolesForPermissionOn(PyObject *self, PyObject *perm, PyObject *object,
-                       PyObject *deflt) {
-	imPermissionRole *im = NULL;
-	PyObject *result;
+c_rolesForPermissionOn(PyObject *perm, PyObject *object, 
+                       PyObject *_default_roles, PyObject *n)
+{
+  PyObject *r, *result = NULL;
 
-	/*| im = imPermissionRole()
-	**|
-	**| im._p="_"+string.translate(perm, name_trans)+"_Permission"
-	**| im._d = default
-	**| return im.__of__(object)
-	*/
+  /*
+    n = n or '_' + string.translate(perm, name_trans) + "_Permission"
+  */
+  if (n)
+    Py_INCREF(n);
+  else
+    {
+      n = permissionName(perm);
+      if (n == NULL)
+        return NULL;
+    }
 
-        im = (imPermissionRole*)PyObject_CallObject(imPermissionRoleObj, NULL);
-	if (im == NULL)
-          return NULL;
+  Py_INCREF(object);
 
-	im->_p = permissionName(perm);
-        if (im->_p == NULL) {
-          Py_DECREF(im);
-          return NULL;
+  /*
+    r = None
+  */
+  r = Py_None;
+  Py_INCREF(r);
+
+  /*
+    while 1:
+  */
+  while (1)
+    {
+      /*
+        if hasattr(object, n):
+            roles = getattr(object, n)
+      */
+      PyObject *roles = PyObject_GetAttr(object, n);
+      if (roles != NULL)
+        {
+
+          /*
+            if roles is None:
+                return 'Anonymous',
+           */
+          if (roles == Py_None)
+            {
+              Py_DECREF(roles);
+              result = AnonymousTuple;
+              Py_INCREF(result);
+              goto end;
+            }
+
+          /*
+            t = type(roles)
+           */
+          /*
+            if t is tuple:
+                # If we get a tuple, then we don't acquire
+                if r is None:
+                    return roles
+                return r+list(roles)
+           */
+          if (PyTuple_Check(roles))
+            {
+              if (r == Py_None)
+                result = roles;
+              else
+                {
+                  PyObject *list_roles = PySequence_List(roles);
+                  Py_DECREF(roles);
+                  if (list_roles == NULL)
+                    goto end;
+                  result = PySequence_Concat(r, list_roles);
+                  Py_DECREF(list_roles);
+                }
+              goto end;
+            }
+
+
+          /*
+            if t is str:
+                # We found roles set to a name.  Start over
+                # with the new permission name.  If the permission
+                # name is '', then treat as private!
+                if roles:
+                    n = roles
+                else:
+                    return _what_not_even_god_should_do
+           */
+          if (PyString_Check(roles))
+            {
+              if (PyString_GET_SIZE(roles))
+                {
+                  Py_DECREF(n);
+                  n = roles;
+                }
+              else
+                {
+                  Py_DECREF(roles);
+                  result = _what_not_even_god_should_do;
+                  Py_INCREF(result);
+                  goto end;
+                }
+            }
+          
+          /*
+            elif roles:
+                if r is None:
+                    r = list(roles)
+                else: r = r + list(roles)
+          */
+          else 
+  	    {
+	      int bool = PyObject_IsTrue(roles);
+	      if (bool < 0)
+		  goto end;
+	      if (bool)
+	        {
+		    PyObject *list_roles = PySequence_List(roles);
+		    Py_DECREF(roles);
+		    if (list_roles == NULL)
+			goto end;
+		    if (r == Py_None)
+		      {
+			  Py_DECREF(r);
+			  r = list_roles;
+		      }
+		    else
+		      {
+			  PyObject *tmp = PySequence_Concat(r, list_roles);
+			  Py_DECREF(list_roles);
+			  if (tmp == NULL)
+			      goto end;
+			  Py_DECREF(r);
+			  r = tmp;
+		      }
+		}
+            }
         }
 
-	if (deflt == NULL) deflt = defaultPermission;
-	im->__roles__ = deflt;
-	Py_INCREF(deflt);
+      else                      /* roles == NULL */
+        PyErr_Clear();
 
-	result = callmethod1(OBJECT(im), __of__, object);
-	Py_DECREF(im);
+      
+      /*
+        object = getattr(object, 'aq_inner', None)
+        if object is None:
+            break
+        object = object.aq_parent
+       */
+      if (! aq_isWrapper(object)) 
+        break;
+      {
+        PyObject *tobj = aq_inner(object);
+        if (tobj == NULL) 
+          goto end;
+        Py_DECREF(object);
+        object = tobj;
+        
+        if (object == Py_None) 
+          break;
 
-	return result;
-}
+        if (! aq_isWrapper(object)) 
+          break;
+        tobj = aq_parent(object);
+        if (tobj == NULL) 
+          goto end;
+        Py_DECREF(object);
+        object = tobj;
+      }
+    }
+
+  /*
+    if r is None:
+        return default
+
+    return r
+  */
+
+  if (r == Py_None)
+    {
+      result = _default_roles;
+      if (result == NULL)
+        result = ManagerTuple;
+      Py_INCREF(result);
+      goto end;
+    }
+
+  Py_INCREF(r);
+  result = r;
+   
+ end:
+  Py_DECREF(n);
+  Py_DECREF(object);
+  Py_DECREF(r);
+  return result;
+}  
 
 
 /*
@@ -2065,6 +1981,7 @@ guarded_getattr(PyObject *inst, PyObject *name, PyObject *default_,
                 PyObject *validate)
 {
   PyObject *v=0, *t=0;
+  int i;
 
   /* if name[:1] != '_': */
   if (PyString_Check(name) || PyUnicode_Check(name)) {
@@ -2101,6 +2018,50 @@ guarded_getattr(PyObject *inst, PyObject *name, PyObject *default_,
       /*
 
         assertion = Containers(type(inst))
+      */
+      t = PyDict_GetItem(ContainerAssertions, OBJECT(inst->ob_type));
+      if (t != NULL)
+        {
+
+      /*
+        if isinstance(assertion, dict):
+            # We got a table that lets us reason about individual
+            # attrs
+            assertion = assertion.get(name)
+            if assertion:
+                # There's an entry, but it may be a function.
+                if callable(assertion):
+                    return assertion(inst, name)
+
+                # Nope, it's boolean
+                return v
+            raise Unauthorized, name
+      */
+          if (PyDict_Check(t))
+            {
+              PyObject *attrv;
+              
+              attrv = PyDict_GetItem(t, name);
+              if (attrv != NULL)
+                {
+                  i=PyObject_IsTrue(attrv);
+                  if (i < 0) goto err;
+                  if (i) 
+                    {
+                      if (attrv->ob_type->tp_call)
+                        {
+                          Py_DECREF(v);
+                          v = callfunction2(attrv, inst, name);
+                          return v;
+                        }
+                      return v;
+                    }
+                }
+              Py_DECREF(v);
+              goto unauth;
+            }
+
+      /*
         if assertion:
             if callable(assertion):
                 factory = assertion(name, v)
@@ -2111,9 +2072,6 @@ guarded_getattr(PyObject *inst, PyObject *name, PyObject *default_,
             return v
 
       */
-      t = PyDict_GetItem(ContainerAssertions, OBJECT(inst->ob_type));
-      if (t != NULL)
-        {
           if (PyCallable_Check(t))
             {
               PyObject *factory;
@@ -2159,6 +2117,7 @@ guarded_getattr(PyObject *inst, PyObject *name, PyObject *default_,
     }
   }
 
+ unauth:
   /* raise Unauthorized, name */
   PyErr_SetObject(Unauthorized, name);
   return NULL;
@@ -2191,7 +2150,7 @@ module_guarded_getattr(PyObject *ignored, PyObject *args)
 static PyObject *
 module_aq_validate(PyObject *ignored, PyObject *args)
 {
-  PyObject *inst, *obj, *name, *v, *validate;
+  PyObject *inst=NULL, *obj=NULL, *name=NULL, *v=NULL, *validate=NULL;
 
   if (unpacktuple5(args, "validate", 0,
                    &inst, &obj, &name, &v, &validate) < 0) return NULL;
@@ -2202,9 +2161,18 @@ static PyObject *
 module_setDefaultBehaviors(PyObject *ignored, PyObject *args)
 {
   PyObject *result = NULL;
-  int own, auth;
+  int own, auth, verbose;
 
-  if (PyArg_ParseTuple(args, "ii:setDefaultBehaviors", &own, &auth)) {
+  if (PyArg_ParseTuple(args, "iii:setDefaultBehaviors", &own, &auth,
+                       &verbose)) {
+    if (verbose) {
+      PyErr_SetString(PyExc_NotImplementedError,
+                      "This security policy implementation does not implement "
+                      "the verbose option.  To enable verbose security "
+                      "exceptions, add 'security-policy-implementation "
+                      "python' to etc/zope.conf.");
+      return NULL;
+    }
     ownerous = own;
     authenticated = authenticated;
     result = Py_None;
@@ -2265,17 +2233,11 @@ void initcAccessControl(void) {
 
 	if (ZopeSecurityPolicy_setup() < 0) return;
 
-	ZopeSecurityPolicyType.tp_getattro =
-		(getattrofunc) PyExtensionClassCAPI->getattro;
-
-	ExtensionClassGetattro= PyExtensionClassCAPI->getattro;
-
-	imPermissionRoleType.tp_getattro =
-		(getattrofunc) PyExtensionClassCAPI->getattro;
+	ExtensionClassGetattro= Py_FindAttr;
 
 	module = Py_InitModule3("cAccessControl",
 		cAccessControl_methods,
-		"$Id: cAccessControl.c,v 1.20.2.22 2005/08/02 09:32:30 andreasjung Exp $\n");
+		"$Id: cAccessControl.c 41697 2006-02-19 12:20:27Z andreasjung $\n");
 
 	aq_init(); /* For Python <= 2.1.1, aq_init() should be after
                       Py_InitModule(). */
@@ -2314,6 +2276,16 @@ void initcAccessControl(void) {
 	Py_DECREF(module);
 	module = NULL;
 
+
+	/*| from ZopeSecurityPolicy import getRoles
+	*/
+
+	IMPORT(module, "AccessControl.ZopeSecurityPolicy");
+	GETATTR(module, getRoles);
+	Py_DECREF(module);
+	module = NULL;
+
+
 	/*| from unauthorized import Unauthorized
 	*/
 
@@ -2330,12 +2302,12 @@ void initcAccessControl(void) {
 	Py_DECREF(module);
 	module = NULL;
 
-	/*| from zLOG import LOG, PROBLEM
+	/*| from logger_wrapper import warn
 	*/
 
-	IMPORT(module, "zLOG");
-	GETATTR(module, LOG);
-	GETATTR(module, PROBLEM);
+	IMPORT(module, "logger_wrapper");
+	GETATTR(module, warn);
 	Py_DECREF(module);
 	module = NULL;
 }
+
