@@ -1,26 +1,30 @@
 ##############################################################################
 #
-# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
+# Copyright (c) 2002 Zope Corporation and Contributors. All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
-# FOR A PARTICULAR PURPOSE
+# FOR A PARTICULAR PURPOSE.
 #
 ##############################################################################
 """Application support
 
-$Id: Application.py 30728 2005-06-10 10:05:01Z shh $
+$Id: Application.py 71171 2006-11-18 10:13:25Z andreasjung $
 """
 
 import os, sys, traceback
+from logging import getLogger
 from cgi import escape
 from StringIO import StringIO
+from warnings import warn
 
-import Globals, Products, App.Product, App.ProductRegistry, misc_
+import Globals, Products, App.Product, App.ProductRegistry
 import transaction
+from Globals import InitializeClass
+from AccessControl import ClassSecurityInfo
 from AccessControl.User import UserFolder
 from Acquisition import aq_base
 from App.ApplicationManager import ApplicationManager
@@ -29,14 +33,19 @@ from App.Product import doInstall
 from App.ProductContext import ProductContext
 from DateTime import DateTime
 from HelpSys.HelpSys import HelpSys
-from misc_ import Misc_
 from webdav.NullResource import NullResource
 from zExceptions import Redirect as RedirectException, Forbidden
-from zLOG import LOG, ERROR, WARNING, INFO
+
+from zope.interface import implements
 
 import Folder
+import misc_
 import ZDOM
 from FindSupport import FindSupport
+from interfaces import IApplication
+from misc_ import Misc_
+
+LOG = getLogger('Application')
 
 class Application(Globals.ApplicationDefaultPermissions,
                   ZDOM.Root, Folder.Folder,
@@ -44,8 +53,11 @@ class Application(Globals.ApplicationDefaultPermissions,
 
     """Top-level system object"""
 
+    implements(IApplication)
+
+    security = ClassSecurityInfo()
+
     title ='Zope'
-    #__roles__=['Manager', 'Anonymous']
     __defined_roles__=('Manager','Anonymous','Owner')
     web__form__method='GET'
     isTopLevelPrincipiaApplicationObject=1
@@ -96,7 +108,7 @@ class Application(Globals.ApplicationDefaultPermissions,
         return self.title
 
     def __class_init__(self):
-        Globals.default__class_init__(self)
+        InitializeClass(self)
 
     def PrincipiaRedirect(self, destination, URL1):
         """Utility function to allow user-controlled redirects"""
@@ -128,7 +140,7 @@ class Application(Globals.ApplicationDefaultPermissions,
 
     ZopeTime = PrincipiaTime
 
-    ZopeAttributionButton__roles__=None
+    security.declarePublic('ZopeAttributionButton')
     def ZopeAttributionButton(self):
         """Returns an HTML fragment that displays the 'powered by zope'
         button along with a link to the Zope site."""
@@ -187,7 +199,7 @@ class Application(Globals.ApplicationDefaultPermissions,
         # We're at the base of the path.
         return ('',)
 
-    fixupZClassDependencies__roles__=()
+    security.declarePrivate('fixupZClassDependencies')
     def fixupZClassDependencies(self, rebuild=0):
         # Note that callers should not catch exceptions from this method
         # to ensure that the transaction gets aborted if the registry
@@ -224,9 +236,7 @@ class Application(Globals.ApplicationDefaultPermissions,
                             ob._register()
                             result=1
                             if not rebuild:
-                                LOG('Zope', INFO,
-                                    'Registered ZClass: %s' % ob.id
-                                    )
+                                LOG.info('Registered ZClass: %s' % ob.id)
                     # Include subobjects.
                     if hasattr(base, 'objectItems'):
                         m = list(ob.objectItems())
@@ -239,13 +249,12 @@ class Application(Globals.ApplicationDefaultPermissions,
                             m = list(ps.methods.objectItems())
                             items.extend(m)
                 except:
-                    LOG('Zope', WARNING,
-                        'Broken objects exist in product %s.' % product.id,
-                        error=sys.exc_info())
+                    LOG.warn('Broken objects exist in product %s.' % product.id,
+                             exc_info=sys.exc_info())
 
         return result
 
-    checkGlobalRegistry__roles__=()
+    security.declarePrivate('checkGlobalRegistry')
     def checkGlobalRegistry(self):
         """Check the global (zclass) registry for problems, which can
         be caused by things like disk-based products being deleted.
@@ -253,27 +262,29 @@ class Application(Globals.ApplicationDefaultPermissions,
         try:
             keys=list(self._p_jar.root()['ZGlobals'].keys())
         except:
-            LOG('Zope', ERROR,
+            LOG.error(
                 'A problem was found when checking the global product '\
                 'registry.  This is probably due to a Product being '\
                 'uninstalled or renamed.  The traceback follows.',
-                error=sys.exc_info())
+                exc_info=sys.exc_info())
             return 1
         return 0
 
-    _setInitializerRegistry__roles__ = ()
+    security.declarePrivate('_setInitializerFlag')
     def _setInitializerFlag(self, flag):
         if self._initializer_registry is None:
             self._initializer_registry = {}
         self._initializer_registry[flag] = 1
 
-    _getInitializerRegistry__roles__ = ()
+    security.declarePrivate('_getInitializerFlag')
     def _getInitializerFlag(self, flag):
         reg = self._initializer_registry
         if reg is None:
             reg = {}
         return reg.get(flag)
 
+InitializeClass(Application)
+
 
 class Expired(Globals.Persistent):
 
@@ -372,10 +383,9 @@ class AppInitializer:
                 mount_paths = [ x[0] for x in dbtab_config.listMountPaths() ]
                 if not '/temp_folder' in mount_paths:
                     # we won't be able to create the mount point properly
-                    LOG('Zope Default Object Creation', ERROR,
-                        ('Could not initialze a Temporary Folder because '
-                         'a database was not configured to be mounted at '
-                         'the /temp_folder mount point'))
+                    LOG.error('Could not initialze a Temporary Folder because '
+                              'a database was not configured to be mounted at '
+                              'the /temp_folder mount point')
                     return
                 try:
                     manage_addMounts(app, ('/temp_folder',))
@@ -383,10 +393,8 @@ class AppInitializer:
                     self.commit('Added temp_folder')
                     tf = app.temp_folder
                 except:
-                    LOG('Zope Default Object Creation', ERROR,
-                        ('Could not add a /temp_folder mount point due to an '
-                        'error.'),
-                        error=sys.exc_info())
+                    LOG.error('Could not add a /temp_folder mount point due to an '
+                              'error.', exc_info=sys.exc_info())
                     return
 
         # Ensure that there is a transient object container in the temp folder
@@ -401,7 +409,7 @@ class AppInitializer:
             default_period_secs = 20
             default_timeout_mins = 20
 
-            limit = getattr(config, 'maximum_number_of_session_objects', 
+            limit = getattr(config, 'maximum_number_of_session_objects',
                             default_limit)
             timeout_spec = getattr(config, 'session_timeout_minutes',
                                    default_timeout_mins)
@@ -409,25 +417,16 @@ class AppInitializer:
                                   default_period_secs)
 
             if addnotify and app.unrestrictedTraverse(addnotify, None) is None:
-                LOG('Zope Default Object Creation', WARNING,
-                    ('failed to use nonexistent "%s" script as '
-                     'session-add-notify-script-path' % addnotify))
+                LOG.warn('failed to use nonexistent "%s" script as '
+                         'session-add-notify-script-path' % addnotify)
                 addnotify=None
 
             if delnotify and app.unrestrictedTraverse(delnotify, None) is None:
-                LOG('Zope Default Object Creation', WARNING,
-                    ('failed to use nonexistent "%s" script as '
-                     'session-delete-notify-script-path' % delnotify))
+                LOG.warn('failed to use nonexistent "%s" script as '
+                         'session-delete-notify-script-path' % delnotify)
                 delnotify=None
 
-            toc = TransientObjectContainer(
-                'session_data', 'Session Data Container',
-                addNotification = addnotify,
-                delNotification = delnotify,
-                limit=limit,
-                period_secs=period_spec)
-
-            if timeout_spec is not None:
+            if 1:  # Preserve indentation for diff
                 toc = TransientObjectContainer('session_data',
                                                'Session Data Container',
                                                timeout_mins = timeout_spec,
@@ -453,6 +452,10 @@ class AppInitializer:
             from Products.Sessions.BrowserIdManager import BrowserIdManager
             bid = BrowserIdManager('browser_id_manager', 'Browser Id Manager')
             app._setObject('browser_id_manager', bid)
+            # FIXME explicitely call manage_afterAdd, as sometimes
+            # events are initialized too late
+            browser_id_manager = app.browser_id_manager
+            browser_id_manager.manage_afterAdd(browser_id_manager, app)
             app._setInitializerFlag('browser_id_manager')
             self.commit('Added browser_id_manager')
 
@@ -469,6 +472,10 @@ class AppInitializer:
                 path='/temp_folder/session_data',
                 requestName='SESSION')
             app._setObject('session_data_manager', sdm)
+            # FIXME explicitely call manage_afterAdd, as sometimes
+            # events are initialized too late
+            session_data_manager = app.session_data_manager
+            session_data_manager.manage_afterAdd(session_data_manager, app)
             app._setInitializerFlag('session_data_manager')
             self.commit('Added session_data_manager')
 
@@ -517,6 +524,10 @@ class AppInitializer:
             from Products.SiteErrorLog.SiteErrorLog import SiteErrorLog
             error_log = SiteErrorLog()
             app._setObject('error_log', error_log)
+            # FIXME explicitely call manage_afterAdd, as sometimes
+            # events are initialized too late
+            error_log = app.error_log
+            error_log.manage_afterAdd(error_log, app)
             app._setInitializerFlag('error_log')
             self.commit('Added site error_log at /error_log')
 
@@ -548,18 +559,18 @@ class AppInitializer:
         bad_things=0
         try:
             if app.checkGlobalRegistry():
-                LOG('Zope', INFO,
+                LOG.info(
                     'Beginning attempt to rebuild the global ZClass registry.')
                 app.fixupZClassDependencies(rebuild=1)
                 did_fixups=1
-                LOG('Zope', INFO,
+                LOG.info(
                     'The global ZClass registry has successfully been rebuilt.')
                 transaction.get().note('Rebuilt global product registry')
                 transaction.commit()
         except:
             bad_things=1
-            LOG('Zope', ERROR, 'The attempt to rebuild the registry failed.',
-                error=sys.exc_info())
+            LOG.error('The attempt to rebuild the registry failed.',exc_info=True)
+               
             transaction.abort()
 
         # Now we need to see if any (disk-based) products were installed
@@ -575,18 +586,15 @@ class AppInitializer:
             # product was added or updated and we are not a ZEO client.
             if getattr(Globals, '__disk_product_installed__', None):
                 try:
-                    LOG('Zope', INFO,
-                        ('New disk product detected, determining if we need '
-                        'to fix up any ZClasses.'))
+                    LOG.info('New disk product detected, determining if we need '
+                             'to fix up any ZClasses.')
                     if app.fixupZClassDependencies():
-                        LOG('Zope',INFO,
-                            'Repaired broken ZClass dependencies.')
+                        LOG.info('Repaired broken ZClass dependencies.')
                         self.commit('Repaired broked ZClass dependencies')
                 except:
-                    LOG('Zope', ERROR,
-                        ('Attempt to fixup ZClass dependencies after '
-                         'detecting an updated disk-based product failed.'),
-                        error=sys.exc_info())
+                    LOG.error('Attempt to fixup ZClass dependencies after '
+                              'detecting an updated disk-based product failed.',
+                              exc_info=sys.exc_info())
                     transaction.abort()
 
     def install_products(self):
@@ -626,7 +634,7 @@ def install_products(app):
                         folder_permissions, raise_exc=debug_mode)
 
     Products.meta_types=Products.meta_types+tuple(meta_types)
-    Globals.default__class_init__(Folder.Folder)
+    InitializeClass(Folder.Folder)
 
 def get_products():
     """ Return a list of tuples in the form:
@@ -663,10 +671,10 @@ def import_products():
 
     for priority, product_name, index, product_dir in products:
         if done.has_key(product_name):
-            LOG('OFS.Application', WARNING, 'Duplicate Product name',
-                'After loading Product %s from %s,\n'
-                'I skipped the one in %s.\n' % (
-                `product_name`, `done[product_name]`, `product_dir`) )
+            LOG.warn('Duplicate Product name: '
+                     'After loading Product %s from %s, '
+                     'I skipped the one in %s.' % (
+                    `product_name`, `done[product_name]`, `product_dir`) )
             continue
         done[product_name]=product_dir
         import_product(product_dir, product_name, raise_exc=debug_mode)
@@ -701,8 +709,8 @@ def import_product(product_dir, product_name, raise_exc=0, log_exc=1):
         except:
             exc = sys.exc_info()
             if log_exc:
-                LOG('Zope', ERROR, 'Could not import %s' % pname,
-                    error=exc)
+                LOG.error('Could not import %s' % pname,
+                          exc_info=exc)
             f=StringIO()
             traceback.print_exc(100,f)
             f=f.getvalue()
@@ -728,7 +736,6 @@ def install_product(app, product_dir, product_name, meta_types,
     path_join=os.path.join
     isdir=os.path.isdir
     exists=os.path.exists
-    DictType=type({})
     global_dict=globals()
     silly=('__doc__',)
 
@@ -749,7 +756,7 @@ def install_product(app, product_dir, product_name, meta_types,
             # like icon images.
             misc_=pgetattr(product, 'misc_', {})
             if misc_:
-                if type(misc_) is DictType:
+                if isinstance(misc_, dict):
                     misc_=Misc_(product_name, misc_)
                 Application.misc_.__dict__[product_name]=misc_
 
@@ -777,6 +784,13 @@ def install_product(app, product_dir, product_name, meta_types,
             # constructors, etc.
             permissions={}
             new_permissions={}
+            if pgetattr(product, '__ac_permissions__', None) is not None:
+                warn('__init__.py of %s has a long deprecated '
+                     '\'__ac_permissions__\' attribute. '
+                     '\'__ac_permissions__\' will be ignored by '
+                     'install_product in Zope 2.11. Please use registerClass '
+                     'instead.' % product.__name__,
+                     DeprecationWarning)
             for p in pgetattr(product, '__ac_permissions__', ()):
                 permission, names, default = (
                     tuple(p)+('Manager',))[:3]
@@ -786,6 +800,12 @@ def install_product(app, product_dir, product_name, meta_types,
                 elif not folder_permissions.has_key(permission):
                     new_permissions[permission]=()
 
+            if pgetattr(product, 'meta_types', None) is not None:
+                warn('__init__.py of %s has a long deprecated \'meta_types\' '
+                     'attribute. \'meta_types\' will be ignored by '
+                     'install_product in Zope 2.11. Please use registerClass '
+                     'instead.' % product.__name__,
+                     DeprecationWarning)
             for meta_type in pgetattr(product, 'meta_types', ()):
                 # Modern product initialization via a ProductContext
                 # adds 'product' and 'permission' keys to the meta_type
@@ -797,6 +817,14 @@ def install_product(app, product_dir, product_name, meta_types,
                 meta_type['visibility'] = 'Global'
                 meta_types.append(meta_type)
 
+            if pgetattr(product, 'methods', None) is not None:
+                warn("__init__.py of %s has a long deprecated 'methods' "
+                     "attribute. 'methods' support might be removed in Zope "
+                     "2.11 or a later feature release. Please use the "
+                     "'legacy' argument of registerClass instead if the "
+                     "methods are constructors. Or refactor the product "
+                     "using adapters." % product.__name__,
+                     DeprecationWarning)
             for name,method in pgetattr(
                 product, 'methods', {}).items():
                 if not hasattr(Folder.Folder, name):
@@ -820,15 +848,15 @@ def install_product(app, product_dir, product_name, meta_types,
                     list(Folder.Folder.__ac_permissions__)+new_permissions)
 
             if not doInstall():
-                transaction().abort()
+                transaction.abort()
             else:
                 transaction.get().note('Installed product '+product_name)
                 transaction.commit()
 
         except:
             if log_exc:
-                LOG('Zope',ERROR,'Couldn\'t install %s' % product_name,
-                    error=sys.exc_info())
+                LOG.error('Couldn\'t install %s' % product_name,
+                           exc_info=sys.exc_info())
             transaction.abort()
             if raise_exc:
                 raise
@@ -858,6 +886,11 @@ def install_standards(app):
                 continue
             app.manage_addProduct['PageTemplates'].manage_addPageTemplate(
                 id=fn, title='', text=open(ob.filename))
+        elif ext in ('.ico', '.gif', '.png'):
+            if hasattr(app, fn):
+                continue
+            app.manage_addProduct['OFSP'].manage_addImage(
+                id=fn, title='', file=open(os.path.join(std_dir, fn)))
         else:
             continue
         wrote = 1
@@ -886,7 +919,7 @@ def reinstall_product(app, product_name):
             break
 
     Products.meta_types=Products.meta_types+tuple(meta_types)
-    Globals.default__class_init__(Folder.Folder)
+    InitializeClass(Folder.Folder)
 
 
 def reimport_product(product_name):
