@@ -1,7 +1,7 @@
 # Authors: David Goodger, Ueli Schlaepfer
 # Contact: goodger@users.sourceforge.net
-# Revision: $Revision: 1.2.10.7 $
-# Date: $Date: 2005/10/09 14:43:46 $
+# Revision: $Revision: 4242 $
+# Date: $Date: 2006-01-06 00:28:53 +0100 (Fri, 06 Jan 2006) $
 # Copyright: This module has been placed in the public domain.
 
 """
@@ -9,10 +9,13 @@ Transforms related to the front matter of a document or a section
 (information found before the main text):
 
 - `DocTitle`: Used to transform a lone top level section's title to
-  the document title, and promote a remaining lone top-level section's
-  title to the document subtitle.
+  the document title, promote a remaining lone top-level section's
+  title to the document subtitle, and determine the document's title
+  metadata (document['title']) based on the document title and/or the
+  "title" setting.
 
-- `SectionTitle`: Used to transform a lone subsection into a subtitle.
+- `SectionSubTitle`: Used to transform a lone subsection into a
+  subtitle.
 
 - `DocInfo`: Used to transform a bibliographic field list into docinfo
   elements.
@@ -195,15 +198,39 @@ class DocTitle(TitlePromoter):
     Any comment elements occurring before the document title or
     subtitle are accumulated and inserted as the first body elements
     after the title(s).
+
+    This transform also sets the document's metadata title
+    (document['title']).
+
+    .. _reStructuredText: http://docutils.sf.net/rst.html
     """
 
     default_priority = 320
 
+    def set_metadata(self):
+        """
+        Set document['title'] metadata title from the following
+        sources, listed in order of priority:
+
+        * Existing document['title'] attribute.
+        * "title" setting.
+        * Document title node (as promoted by promote_title).
+        """
+        if not self.document.hasattr('title'):
+            if self.document.settings.title is not None:
+                self.document['title'] = self.document.settings.title
+            elif len(self.document) and isinstance(self.document[0], nodes.title):
+                self.document['title'] = self.document[0].astext()
+
     def apply(self):
-        if not getattr(self.document.settings, 'doctitle_xform', 1):
-            return
-        if self.promote_title(self.document):
-            self.promote_subtitle(self.document)
+        if getattr(self.document.settings, 'doctitle_xform', 1):
+            # promote_(sub)title defined in TitlePromoter base class.
+            if self.promote_title(self.document):
+                # If a title has been promoted, also try to promote a
+                # subtitle.
+                self.promote_subtitle(self.document)
+        # Set document['title'].
+        self.set_metadata()
 
 
 class SectionSubTitle(TitlePromoter):
@@ -236,8 +263,7 @@ class SectionSubTitle(TitlePromoter):
     def apply(self):
         if not getattr(self.document.settings, 'sectsubtitle_xform', 1):
             return
-        for section in self.document.traverse(lambda n:
-                                              isinstance(n, nodes.section)):
+        for section in self.document.traverse(nodes.section):
             # On our way through the node tree, we are deleting
             # sections, but we call self.promote_subtitle for those
             # sections nonetheless.  To do: Write a test case which
@@ -277,7 +303,7 @@ class DocInfo(Transform):
                         Status
                     <field_body>
                         <paragraph>
-                            $RCSfile: frontmatter.py,v $
+                            $RCSfile$
             ...
 
     After running the bibliographic field list transform, the
@@ -315,6 +341,10 @@ class DocInfo(Transform):
       can't prevent any RCS keywords used in this explanation from
       being expanded. Only the "RCSfile" keyword is stable; its
       expansion text changes only if the file name changes.)
+
+    .. _reStructuredText: http://docutils.sf.net/rst.html
+    .. _reStructuredText Markup Specification:
+       http://docutils.sf.net/docs/ref/rst/restructuredtext.html
     """
 
     default_priority = 340
@@ -421,8 +451,8 @@ class DocInfo(Transform):
         return 1
 
     rcs_keyword_substitutions = [
-          (re.compile(r'\$' r'Date: (\d\d\d\d)/(\d\d)/(\d\d) [\d:]+ \$',
-                      re.IGNORECASE), r'\1-\2-\3'),
+          (re.compile(r'\$' r'Date: (\d\d\d\d)[-/](\d\d)[-/](\d\d)[ T][\d:]+'
+                      r'[^$]* \$', re.IGNORECASE), r'\1-\2-\3'),
           (re.compile(r'\$' r'RCSfile: (.+),v \$', re.IGNORECASE), r'\1'),
           (re.compile(r'\$[a-zA-Z]+: (.+) \$'), r'\1'),]
 
@@ -439,10 +469,8 @@ class DocInfo(Transform):
                 authors = self.authors_from_paragraphs(field)
             authornodes = [nodes.author('', '', *author)
                            for author in authors if author]
-            if len(authornodes) > 1:
+            if len(authornodes) >= 1:
                 docinfo.append(nodes.authors('', *authornodes))
-            elif len(authornodes) == 1:
-                docinfo.append(authornodes[0])
             else:
                 raise TransformError
         except TransformError:
